#include "AuthHelper.h"

#include <stdio.h>
#include <stdlib.h>
#include <QtCore/QCoreApplication>
#include <QtCore/QTimer>
#include <QtDBus/QtDBus>
#include <QDebug>
#include <View/ConfigDialog.h>
#include <Config.h>

#include <QDBusServiceWatcher>
#include "AuthcodeUpdateDialog.h"
#include <CloudPrintClient.h>

AuthHelper::AuthHelper(){
    authAgent.connect(&authAgent, SIGNAL(requestAuthCode(QString)), this, SLOT(requireAuthCode(QString)));
    authAgent.connect(&authAgent, SIGNAL(jobFinish(QString,QString,QString)),
                      this , SLOT(handleJobFinish(QString,QString,QString)));

    QDBusServiceWatcher *watcher = new QDBusServiceWatcher("com.deepin.cloudprint.Auth.Agent", QDBusConnection::systemBus(),QDBusServiceWatcher::WatchForRegistration);
    connect(watcher, SIGNAL(serviceRegistered(QString)), this, SLOT(ServiceRegister(QString)));
}

void AuthHelper::ServiceRegister(const QString & /*service*/) {
    authAgent.disconnect(SIGNAL(requestAuthCode(QString)));
    authAgent.disconnect(SIGNAL(jobFinish(QString,QString,QString)));
    authAgent.connect(&authAgent, SIGNAL(requestAuthCode(QString)), this, SLOT(requireAuthCode(QString)));
    authAgent.connect(&authAgent, SIGNAL(jobFinish(QString,QString,QString)),
                      this , SLOT(handleJobFinish(QString,QString,QString)));
}

void AuthHelper::requireAuthCode(const QString &addr) {
    AuthcodeUpdateDialog dialog(addr);
    dialog.show();

    QEventLoop loop;
    connect(&dialog, SIGNAL(closed()), &loop, SLOT(quit()));
    loop.exec();

    Config cfg;
    QUrl scAddr = serverAddress(formatAddress(addr));
    ServerConfig sc = cfg.get(scAddr.toString());

    if (dialog.isContinue()) {
        QString authcode = dialog.getAuthcode();
        qDebug()<<"GET authcode"<<authcode;
        sc.Authcode = authcode;
        cfg.set(scAddr.toString(), sc);
        cfg.save();
    }
    authAgent.SetAuthCode(addr, sc.Authcode);
}

void AuthHelper::handleJobFinish(const QString & /*printerID*/,const QString &job,const QString & error) {
    qDebug()<<"handleJobFinish"<<job<<error;
    QString jobName = job;
    if (jobName.length() >= 14) {
        jobName = jobName.left(11) + "...";
    }

    QString title = tr("Successful Print");
    QString hits = QString(tr("%1 printed successfully\nPlease take them away timely")).arg(jobName);
    if (!error.isEmpty()) {
        title = tr("Print Failure");
        hits = QString(tr("Print jobs are terminated, please contact the admin"));
    }

    QList<QVariant> argumentList;
    argumentList << "deepin-cloud-print-auth-helper";
    argumentList << (uint)0;
    argumentList << "deepin-cloudprint-config-helper";
    argumentList << title;
    argumentList << hits;
    argumentList << QStringList();
    argumentList << QVariantMap();
    argumentList << (int)5000;

    static QDBusInterface notifyApp("org.freedesktop.Notifications",
      "/org/freedesktop/Notifications", "org.freedesktop.Notifications");
    QDBusMessage reply = notifyApp.callWithArgumentList(QDBus::AutoDetect,
      "Notify", argumentList);
    if(reply.type() == QDBusMessage::ErrorMessage) {
      qDebug() << "D-Bus Error:" << reply.errorMessage();
    }
}
