#include "CUPSClient.h"

#include <QUrl>
#include <QProcess>
#include "CUPSClient.h"

#include <QDebug>

bool checkProcessRun(QProcess& app) {
    qDebug()<<"Start "<<app.program()<<app.arguments().join(" ");
    app.start();
    if (!app.waitForStarted()) {
        qDebug()<<"Start " <<"failed";
        return false;
    }

    app.waitForFinished(-1);

    if (QProcess::NormalExit != app.exitStatus()) {
        qDebug()<<app.program()<<"exit status"<<app.exitStatus();
        return false;
    }

    if (0 != app.exitCode()) {
        qDebug()<<app.program()<<"exit with"<<app.exitCode()
                <<app.readAllStandardError()
                <<app.readAllStandardOutput();
        return false;
    }
    return true;
}

QString printerNameEncode(const QString& printerName) {
    return QString(printerName).replace(QRegExp("[\01-\040\0127/#]"), "-").left(128);
}

bool enablePrinter(const QString &printerID) {
    QProcess cupsenable;
    cupsenable.setProgram("/usr/sbin/cupsenable");
    QStringList args;
    args.append(printerNameEncode(printerID));
    cupsenable.setArguments(args);
    return checkProcessRun(cupsenable);
}

bool acceptPrinter(const QString &printerID) {
    QProcess cupsaccept;
    cupsaccept.setProgram("/usr/sbin/cupsaccept");
    QStringList args;
    args.append(printerNameEncode(printerID));
    cupsaccept.setArguments(args);
    return checkProcessRun(cupsaccept);
}

CUPSClient::CUPSClient(QObject *parent) : QObject(parent) {

}

cloudprint::Printer copyCUPSDest(cups_dest_t* dest) {
    cloudprint::Printer printer;
    QMap<QString, QString> options;
    for (int j =0; j < dest->num_options; j ++) {
        options.insert(dest->options[j].name,dest->options[j].value);
    }
    printer.Addr = options["device-uri"].toStdString();
    return printer;
}

QMap<QString, cloudprint::Printer> CUPSClient::listPrinter() {
    QMap<QString, cloudprint::Printer> printers;

    cups_dest_t *dests;
    int num_dests = cupsGetDests(&dests);
    cups_dest_t *dest;
    int i;
    for (i = num_dests, dest = dests; i > 0; i--, dest ++) {
        cloudprint::Printer printer = copyCUPSDest(dest);
        qDebug()<<printer.Addr.c_str();
        printers.insert(printer.Addr.c_str(), printer);
    }
    return printers;
}


cups_dest_t* CUPSClient::getPrinter(const cloudprint::Printer &printer) {
    QUrl url(printer.Addr.c_str());
    QString printerID = QString::fromStdString(printer.ID);
    QString printerName = QString("%1@%2:%3")
            .arg(printerNameEncode(printerID))
            .arg(url.host()).arg(url.port());

    cups_dest_t *dests;
    int num_dests = cupsGetDests(&dests);
    return  cupsGetDest(printerName.toStdString().c_str(), NULL, num_dests, dests);
}

bool CUPSClient::addPrinter(const cloudprint::Printer &printer) {
    qDebug()<<"addPrinter";
    QProcess lpadmin;
    lpadmin.setProgram("/usr/sbin/lpadmin");

    QUrl url(printer.Addr.c_str());
    QString printerID = QString::fromStdString(printer.ID);
    QString printerName = QString("%1@%2:%3")
            .arg(printerNameEncode(printerID))
            .arg(url.host()).arg(url.port());
    QStringList args;
    args.append("-E");
    args.append("-p");
    args.append(printerName);
    args.append("-v");
    args.append(url.toString(QUrl::FullyEncoded));
    args.append("-m");
    args.append("lsb/usr/dcp/DCP.ppd");
    args.append("-o");
    args.append("printer-is-shared=no");
    args.append("-o");
    args.append("printer-error-policy=retry-job");

    lpadmin.setArguments(args);

    return checkProcessRun(lpadmin)
            && enablePrinter(printerName)
            && acceptPrinter(printerName);
}

