#include "CloudPrintClient.h"

#include "RPC/CloudPrint.h"
#include "RPC/Print_constants.h"

#include <thrift/protocol/TBinaryProtocol.h>
#include <thrift/transport/TSocket.h>
#include <thrift/transport/TTransportUtils.h>

#include <QDebug>
#include <QUrl>

using namespace apache::thrift;
using namespace apache::thrift::protocol;
using namespace apache::thrift::transport;

Printer::PrinterStatus Printer::localStatus() {
    if (Status & cloudprint::g_Print_constants.PrinterStatusPaused) {
        return Printer::OutWork;
    }
    if (Status & cloudprint::g_Print_constants.PrinterStatusError) {
        return Printer::OutWork;
    }
    if (Status & cloudprint::g_Print_constants.PrinterStatusPendingDeletion) {
      return Printer::Idle;
    }
    if (Status & cloudprint::g_Print_constants.PrinterStatusPaperJam) {
      return Printer::OutWork;
    }
    if (Status & cloudprint::g_Print_constants.PrinterStatusPaperOut) {
      return Printer::OutWork;
    }
    if (Status & cloudprint::g_Print_constants.PrinterStatusManualFeed) {
      return Printer::OutWork;
    }
    if (Status & cloudprint::g_Print_constants.PrinterStatusPaperProblem) {
      return Printer::OutWork;
    }
    if (Status & cloudprint::g_Print_constants.PrinterStatusOffline) {
      return Printer::OutWork;
    }
    if (Status & cloudprint::g_Print_constants.PrinterStatusIoActive) {
      return Printer::Idle;
    }
    if (Status & cloudprint::g_Print_constants.PrinterStatusBusy) {
      return Printer::Processing;
    }
    if (Status & cloudprint::g_Print_constants.PrinterStatusPrinting) {
      return Printer::Processing;
    }
    if (Status & cloudprint::g_Print_constants.PrinterStatusOutputBinFull) {
      return Printer::OutWork;
    }
    if (Status & cloudprint::g_Print_constants.PrinterStatusNotAvailable) {
      return Printer::OutWork;
    }
    if (Status & cloudprint::g_Print_constants.PrinterStatusWaiting) {
      return Printer::Idle;
    }
    if (Status & cloudprint::g_Print_constants.PrinterStatusProcessing) {
      return Printer::Processing;
    }
    if (Status & cloudprint::g_Print_constants.PrinterStatusInitializing) {
      return Printer::Idle;
    }
    if (Status & cloudprint::g_Print_constants.PrinterStatusWarmingUp) {
      return Printer::Idle;
    }
    if (Status & cloudprint::g_Print_constants.PrinterStatusTonerLow) {
      return Printer::OutWork;
    }
    if (Status & cloudprint::g_Print_constants.PrinterStatusNoToner) {
      return Printer::OutWork;
    }
    if (Status & cloudprint::g_Print_constants.PrinterStatusPagePunt) {
      return Printer::OutWork;
    }
    if (Status & cloudprint::g_Print_constants.PrinterStatusUserIntervention) {
      return Printer::OutWork;
    }
    if (Status & cloudprint::g_Print_constants.PrinterStatusOutOfMemory) {
      return Printer::OutWork;
    }
    if (Status & cloudprint::g_Print_constants.PrinterStatusDoorOpen) {
      return Printer::OutWork;
    }
    if (Status & cloudprint::g_Print_constants.PrinterStatusServerUnknown) {
      return Printer::OutWork;
    }
    if (Status & cloudprint::g_Print_constants.PrinterStatusPowerSave) {
      return Printer::Idle;
    }
    if (Status & cloudprint::g_Print_constants.PrinterStatusServerOffline) {
      return Printer::OutWork;
    }
    if (Status & cloudprint::g_Print_constants.PrinterStatusDriverUpdateNeeded) {
      return Printer::OutWork;
    }
    return Printer::Idle;
}


QString Printer::statusString() {
    if (Status & cloudprint::g_Print_constants.PrinterStatusPaused) {
        return QObject::tr("Paused");
    }
    if (Status & cloudprint::g_Print_constants.PrinterStatusError) {
        return QObject::tr("Unknown error");
    }
    if (Status & cloudprint::g_Print_constants.PrinterStatusPendingDeletion) {
      return QObject::tr("Printer is idle");
    }
    if (Status & cloudprint::g_Print_constants.PrinterStatusPaperJam) {
      return QObject::tr("Paper jammed");
    }
    if (Status & cloudprint::g_Print_constants.PrinterStatusPaperOut) {
      return QObject::tr("Out of paper");
    }
    if (Status & cloudprint::g_Print_constants.PrinterStatusManualFeed) {
      return QObject::tr("Manual feed");
    }
    if (Status & cloudprint::g_Print_constants.PrinterStatusPaperProblem) {
      return QObject::tr("Paper problem");
    }
    if (Status & cloudprint::g_Print_constants.PrinterStatusOffline) {
      return QObject::tr("Offline");
    }
    if (Status & cloudprint::g_Print_constants.PrinterStatusIoActive) {
      return QObject::tr("Printer is idle");
    }
    if (Status & cloudprint::g_Print_constants.PrinterStatusBusy) {
      return QObject::tr("Processing");
    }
    if (Status & cloudprint::g_Print_constants.PrinterStatusPrinting) {
      return QObject::tr("Processing");
    }
    if (Status & cloudprint::g_Print_constants.PrinterStatusOutputBinFull) {
      return QObject::tr("Output bin full");
    }
    if (Status & cloudprint::g_Print_constants.PrinterStatusNotAvailable) {
      return QObject::tr("Unavailable");
    }
    if (Status & cloudprint::g_Print_constants.PrinterStatusWaiting) {
      return QObject::tr("Printer is idle");
    }
    if (Status & cloudprint::g_Print_constants.PrinterStatusProcessing) {
      return QObject::tr("Processing");
    }
    if (Status & cloudprint::g_Print_constants.PrinterStatusInitializing) {
      return QObject::tr("Printer is idle");
    }
    if (Status & cloudprint::g_Print_constants.PrinterStatusWarmingUp) {
      return QObject::tr("Printer is idle");
    }
    if (Status & cloudprint::g_Print_constants.PrinterStatusTonerLow) {
      return QObject::tr("Low on toner");
    }
    if (Status & cloudprint::g_Print_constants.PrinterStatusNoToner) {
      return QObject::tr("No toner");
    }
    if (Status & cloudprint::g_Print_constants.PrinterStatusPagePunt) {
      return QObject::tr("Page punt");
    }
    if (Status & cloudprint::g_Print_constants.PrinterStatusUserIntervention) {
      return QObject::tr("User intervention required");
    }
    if (Status & cloudprint::g_Print_constants.PrinterStatusOutOfMemory) {
      return QObject::tr("Out of memory");
    }
    if (Status & cloudprint::g_Print_constants.PrinterStatusDoorOpen) {
      return QObject::tr("Door opened");
    }
    if (Status & cloudprint::g_Print_constants.PrinterStatusServerUnknown) {
      return QObject::tr("Unknown service");
    }
    if (Status & cloudprint::g_Print_constants.PrinterStatusPowerSave) {
      return QObject::tr("Printer is idle");
    }
    if (Status & cloudprint::g_Print_constants.PrinterStatusServerOffline) {
      return QObject::tr("Server Offline");
    }
    if (Status & cloudprint::g_Print_constants.PrinterStatusDriverUpdateNeeded) {
      return QObject::tr("Need update driver");
    }
    return QObject::tr("Printer is idle");
}

class ClientData {
public:
    QUrl                            *url;
    cloudprint::CloudPrintClient    *client;
    boost::shared_ptr<TTransport>   transport;
    boost::shared_ptr<TSocket>      socket;
};

CloudPrintRPCClient::CloudPrintRPCClient(const QString& addr, QObject *parent) : QObject(parent) {
    qDebug()<<addr;
    data = new ClientData;
    data->url = new QUrl(addr);
    data->socket = boost::shared_ptr<TSocket>(new TSocket(data->url->host().toStdString(), data->url->port(9100)));
    data->socket->setConnTimeout(5000);
    data->transport = boost::shared_ptr<TTransport>(new TBufferedTransport(data->socket));
    boost::shared_ptr<TProtocol> protocol(new TBinaryProtocol(data->transport));
    data->client = new cloudprint::CloudPrintClient(protocol);
}

CloudPrintRPCClient::~CloudPrintRPCClient(){
    if (data) {
        delete data->url;
        delete data->client;
        delete data;
    }
}

bool CloudPrintRPCClient::Connect() {
    bool connect = false;
    try {
        data->transport->open();
        data->transport->close();
        connect = true;
    } catch (TTransportException& e) {
        qDebug()<<"Connect Exception"<<e.what();
    } catch (TApplicationException& e) {
        qDebug()<<"Connect TApplicationException"<<e.what();
    } catch(...) {
        data->transport->close();
        qDebug()<<"Unknow Exception";
    }
    return connect;
}

bool CloudPrintRPCClient::Verfiy(const cloudprint::Auth& auth) {
    bool verfiy = false;
    try {
        data->transport->open();
        verfiy = data->client->Verfiy(auth);
        data->transport->close();
    } catch (TTransportException& e) {
        qDebug()<<"Verfiy Exception"<<e.what();
    } catch (TApplicationException& e) {
        qDebug()<<"TApplicationException"<<e.what();
    }  catch(...) {
        data->transport->close();
        qDebug()<<"Unknow Exception";
    }
    return verfiy;
}

std::vector<Printer> CloudPrintRPCClient::ListPrinter(const cloudprint::Auth& auth){
    std::vector<Printer> printerList;

    try {
        std::vector<cloudprint::Printer> cloudprinterList;
        data->transport->open();
        data->client->ListPrinter(cloudprinterList, auth);
        data->transport->close();
        std::vector<cloudprint::Printer>::iterator itor = cloudprinterList.begin();
        for (; itor != cloudprinterList.end(); ++itor) {
            printerList.push_back(Printer(*itor));
        }
    } catch (TTransportException& e) {
        qDebug()<<"TTransportException"<<e.what();
        throw e;
    } catch (TApplicationException& e) {
        qDebug()<<"TApplicationException"<<e.what();
        throw e;
    }  catch(...) {
        data->transport->close();
        qDebug()<<"Unknow Exception";
        throw;
    }
    QString serverURL = QString("dcp://%1:%2/%3").arg(data->url->host()).arg(data->url->port());
    std::vector<Printer>::iterator itor = printerList.begin();
    for (; itor != printerList.end(); ++itor) {
        itor->Addr = QUrl(serverURL.arg(QString::fromStdString(itor->ID)))
                        .toString(QUrl::FullyEncoded).toStdString();
    }
    return printerList;
}
