#include "Config.h"

#include <QStandardPaths>
#include <QJsonDocument>
#include <QJsonObject>
#include <QDir>
#include <QDebug>

static QString ConfgPath = ".config/deepin/cloud-print";
static QString ConfgFilename = "server.json";
static QString KeyAuthcode = "Authcode";
static QString KeyAddress = "Address";
static QString KeyLastUsed = "LastUsed";

Config::Config() {
    load();
}

void Config::load() {
    QString homeDir = QStandardPaths::standardLocations(QStandardPaths::HomeLocation).at(0);
    QFile file(QString("%1/%2/%3").arg(homeDir).arg(ConfgPath).arg(ConfgFilename));
    file.open(QIODevice::ReadOnly | QIODevice::Text);
    QString val = file.readAll();
    file.close();

    QJsonDocument d = QJsonDocument::fromJson(val.toUtf8());
    QJsonObject cfgObj = d.object();
    QStringList servers = cfgObj.keys();
    foreach (QString saddr, servers) {
        QJsonObject serverCfg = cfgObj.value(saddr).toObject();
        ServerConfig sc;
        sc.Address =   saddr;
        sc.Authcode = serverCfg.value(KeyAuthcode).toString();
        sc.LastUsed = QDateTime::fromString(serverCfg.value(KeyLastUsed).toString());
        config.insert(saddr, sc);
    }
}

ServerConfig Config::lastUsed() {
    QMap<QString, ServerConfig>::Iterator itor = config.begin();
    ServerConfig lastUsed;
    if (itor != config.end()) {
        lastUsed = itor.value();
    }
    for (; itor != config.end(); ++itor) {
        if (itor.value().LastUsed > lastUsed.LastUsed) {
            lastUsed = itor.value();
        }
    }
    return lastUsed;
}

void Config::save() {
    QDir homeDir = QDir(QStandardPaths::standardLocations(QStandardPaths::HomeLocation).at(0));
    homeDir.mkpath(ConfgPath);
    QFile file(QString("%1/%2/%3").arg(homeDir.path()).arg(ConfgPath).arg(ConfgFilename));
    file.open(QIODevice::WriteOnly | QIODevice::Truncate);

    QJsonObject cfgObj;
    QMap<QString, ServerConfig>::Iterator itor = config.begin();
    for (; itor != config.end(); ++itor) {
        QJsonObject serverObj;
        serverObj.insert(KeyAuthcode, itor.value().Authcode);
        serverObj.insert(KeyAddress, itor.value().Address);
        serverObj.insert(KeyLastUsed, itor.value().LastUsed.toString());
        cfgObj.insert(itor.key(), serverObj);
    }

    QJsonDocument doc(cfgObj);
    //qDebug()<<QString(doc.toJson());
    file.write(doc.toJson());
    file.close();
}

