#include "Utils.h"

#include <iostream>

#include <QApplication>
#include <QTextStream>
#include <QFile>
#include <QDir>
#include <QStandardPaths>
#include <QDebug>
#include <QTranslator>

namespace Utils {

QString g_AppName = "deepin-cloud-print-config";
QString g_LogPath;
QString g_LogDir;

void crashMessageOutput(QtMsgType type, const QMessageLogContext &c, const QString & str) {
    QString txt;
    switch(type) {
    case QtDebugMsg:
        txt = QString("|-[DEBUG] %1:%2\n |-%3").arg(c.function).arg(c.line).arg(str);
        break;
#if QT_VERSION >= 0x050500
    case QtInfoMsg:
        txt = QString("|-[INFO] %1:%2\n |-%3").arg(c.function).arg(c.line).arg(str);
        break;
#endif
    case QtWarningMsg:
        txt = QString("|-[WARNING]%1:%2\n |-%3").arg(c.function).arg(c.line).arg(str);
        break;
    case QtCriticalMsg:
        txt = QString("|-[CRITICAL]%1:%2\n |-%3").arg(c.function).arg(c.line).arg(str);
        break;
    case QtFatalMsg:
        txt = QString("|-[FATAL]%1:%2\n |-%3").arg(c.function).arg(c.line).arg(str);
        abort();
    default:
        txt = QString("|-[WARNING]%1:%2\n |-%3").arg(c.function).arg(c.line).arg(str);
        break;
    }

    QFile outFile(g_LogPath);
    outFile.open(QIODevice::Text | QIODevice::WriteOnly | QIODevice::Append);
    QTextStream ts(&outFile);
    ts << txt << endl;

    std::wcout<<txt.toStdWString()<<std::endl;
}

void installLogHandler() {
    QDir tmpDir(QStandardPaths::standardLocations(QStandardPaths::TempLocation).first());
    tmpDir.mkdir(g_AppName);
    g_LogDir = tmpDir.absoluteFilePath(g_AppName);
    g_LogPath = QDir::toNativeSeparators(QDir(g_LogDir).absoluteFilePath("dcpc.log"));
    qDebug() << "Install Log to " << g_LogPath;
    qInstallMessageHandler(crashMessageOutput);
}

void loadTranslate() {
    QCoreApplication& app = *QApplication::instance();
    QTranslator *translator = new QTranslator();
    QString tnapplang;
    QString tnappcoun;
    QString clangcode = "";
    QStringList allappargs = app.arguments();
    QList<QPair<QString, QString> > oppairs;

    for(QList<QString>::const_iterator i = allappargs.constBegin(); i < allappargs.constEnd(); ++i) {
        if(i->count('=') == 1) {
            oppairs.append(QPair<QString, QString> (i->section('=', 0, 0).simplified(), i->section('=', 1, 1).simplified()));
        }
    }

    for(QList<QPair<QString, QString> >::const_iterator i = oppairs.constBegin(); i < oppairs.constEnd(); ++i) {
        if(i->first.contains("lang", Qt::CaseInsensitive)) {
            clangcode = i->second;
            tnapplang = clangcode.left(2);

            if(clangcode.contains('_') && clangcode.size() == 5) {
                tnappcoun = clangcode.section('_', -1, -1);
            }

            break;
        }
    }

    if(clangcode.isEmpty()) {
        clangcode = QLocale::system().name();
        tnapplang = clangcode.left(2);

        if(clangcode.contains('_') && clangcode.size() == 5) {
            tnappcoun = clangcode.section('_', -1, -1);
        }
    }

    QString tranlateUrl;

    if (!tnappcoun.isEmpty()) {
        tranlateUrl = QString(":/po/%1_%2.qm").arg(tnapplang).arg(tnappcoun);
    }

    if (!QFile::exists(tranlateUrl)) {
        tranlateUrl = QString(":/po/%1.qm").arg(tnapplang);
    }


    if(!QFile::exists(tranlateUrl)) {
        tranlateUrl = ":/en_US.qm";
    }

    qDebug()<<"Load translate file: "<<tranlateUrl;

    if(translator->load(tranlateUrl)) {
        app.installTranslator(translator);
    }
}

}

