#include "ConfigDialog.h"

#include <QUrl>
#include <QTimer>
#include <QColor>
#include <QThread>
#include <QLineEdit>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QPushButton>
#include <QProcess>
#include <QEvent>
#include <QKeyEvent>
#include <QApplication>

#include "DTitle.h"
#include "DLabel.h"
#include "DLineEdit.h"
#include "DPushButton.h"
#include "DListWidget.h"
#include "DPasswordEdit.h"
#include "DSpinner.h"
#include <Dialog/DMessageBox.h>
#include "PrinterWidget.h"
#include "../CloudPrintClient.h"
#include "../CUPSClient.h"
#include "../Config.h"

#include <QDebug>

static QString DCPSchema = "dcp";

QUrl formatAddress(QString rawAddr) {
    QUrl url;
    if (rawAddr.contains("dcp://")) {
        url.setUrl(rawAddr);
        url.setPort(url.port(9100));
        return url;
    }

    QString formatAddr = QString("%1://%2:%3").arg(DCPSchema);
    QStringList addrs = rawAddr.split(":");
    if (addrs.length() < 1) {
        return url;
    }

    formatAddr = formatAddr.arg(addrs.at(0));

    if (addrs.length() >= 2) {
        formatAddr = formatAddr.arg(addrs.at(1));
    }else{
        formatAddr = formatAddr.arg(9100);
    }

    url.setUrl(formatAddr);

    return url;
}

QUrl serverAddress(QUrl formatUrl) {
    QUrl serverUrl;
    serverUrl.setScheme(formatUrl.scheme());
    serverUrl.setHost(formatUrl.host());
    serverUrl.setPort(formatUrl.port());
    return serverUrl;
}

void ConfigDialogInnter::Worker::doListPrinter() {
    QThread::sleep(1);
    QUrl url = serverAddress(formatAddress(ipaddr));
    QString addr = url.toString();

    client = new CloudPrintRPCClient(addr,this);
    auth = new cloudprint::Auth();
    auth->Authcode = authcode.toStdString();

    if (!client->Connect()) {
        emit listPrinterFinish(InvaildAddress);
        return;
    }

    if (!client->Verfiy(*auth)) {
        emit listPrinterFinish(InvaildAuthcode);
        return;
    }

    ServerConfig sc;
    sc.Address = addr;
    sc.Authcode = auth->Authcode.c_str();
    sc.LastUsed = QDateTime::currentDateTime();

    Config cfg;
    cfg.config[sc.Address]= sc;
    cfg.save();

    try {
        printers = listPrinter();
    }catch(...) {
        qDebug()<<"listPrinter failed";
        emit listPrinterFinish(InvaildPrinterList);
        return;
    }

    emit listPrinterFinish(Success);
}

std::vector<Printer> ConfigDialogInnter::Worker::listPrinter() {
    return client->ListPrinter(*auth);
}

class ConfigData {
public:
    DTitle          *title;
    DLineEdit       *ipedit;
    DLabel          *ipHits;
    DLabel          *codeHits;
    DPasswordEdit   *authcodeEdit;
    DPushButton     *actBt;
    QWidget         *authWidget;
    QWidget         *listWidget;
    QVBoxLayout     *layout;
    DSpinner        *spinner;

    QProcess        *dManual;
    bool            verfiy;
    bool            quitAfterVerfiy;
    QString         authcode;

    ConfigDialogInnter::Worker    *worker;
};

ConfigDialog::ConfigDialog(QWidget *parent) : DWindow(parent) {
    data = new ConfigData;
    data->dManual = NULL;
    data->quitAfterVerfiy = false;
    resize(520, 356);

    QVBoxLayout *layout = new QVBoxLayout;
    layout->setContentsMargins(10,10,10,10);
    layout->setSpacing(0);

    data->title = new DTitle;
    data->title->setFixedSize(500, 36);
    connect(data->title, SIGNAL(minimised()),
            this, SLOT(showMinimized()));
    connect(data->title, SIGNAL(closed()),
            this, SLOT(close()));
    connect(data->title, SIGNAL(closed()),
            this, SIGNAL(closed()));

    layout->addWidget(data->title);

    data->title->setText(tr("Add Cloud Printer"));
    data->authWidget = createAuthWidget();
    layout->addWidget(data->authWidget);
    data->layout = layout;
    setLayout(layout);

    QThread* thread = new QThread;
    data->worker = new ConfigDialogInnter::Worker();
    data->worker->moveToThread(thread);
    connect(data->worker, SIGNAL(listPrinterFinish(int)), this, SLOT(finishVerfiy(int)));
    connect(this, SIGNAL(verfiyStart()), data->worker, SLOT(doListPrinter()));
    connect(data->worker, SIGNAL(finished()), thread, SLOT(quit()));
    connect(data->worker, SIGNAL(finished()), data->worker, SLOT(deleteLater()));
    connect(thread, SIGNAL(finished()), thread, SLOT(deleteLater()));
    thread->start();

    Config cfg;
    ServerConfig sc = cfg.lastUsed();
    if (!sc.Address.isEmpty()) {
        data->ipedit->setText(QUrl(sc.Address).host());
        data->authcodeEdit->setText(sc.Authcode);
    }

    qApp->installEventFilter(this);
}

void ConfigDialog::clearHits() {
    data->ipHits->setText("");
    data->codeHits->setText("");
}

QWidget *ConfigDialog::createAuthWidget() {
    QVBoxLayout *layout = new QVBoxLayout;
    layout->setSpacing(0);
    layout->setMargin(0);

    QHBoxLayout *ipLayout = new QHBoxLayout;
    DLabel *ipLabel = new DLabel(tr("IP Address"));
    ipLabel->setFixedSize(120,36);
    ipLabel->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
    DLineEdit *ipedit = new DLineEdit();
    ipedit->setPlaceholderText(tr("Please input the printer IP address"));
    ipedit->setFixedSize(220,36);
    ipLayout->setSpacing(16);
    ipLayout->addStretch();
    ipLayout->addWidget(ipLabel);
    ipLayout->addWidget(ipedit);
    ipLayout->addSpacing(44);
    ipLayout->addStretch();
    connect(ipedit, &DLineEdit::textChanged, [this](const QString&) {clearHits();});

    QString hitsSytleSheet = "color : #ff7f00;";
    QHBoxLayout *ipHitsLayout = new QHBoxLayout;
    data->ipHits = new DLabel();
    data->ipHits->setFixedSize(220,36);
    data->ipHits->setStyleSheet(hitsSytleSheet);
    ipHitsLayout->setSpacing(16);
    ipHitsLayout->addStretch();
    ipHitsLayout->addSpacing(140);
    ipHitsLayout->addWidget(data->ipHits);
    ipHitsLayout->addSpacing(34);
    ipHitsLayout->addStretch();

    QHBoxLayout *codeLayout = new QHBoxLayout;
    DLabel *codeLabel = new DLabel(tr("Authorization Code"));
    codeLabel->setFixedSize(120,36);
    codeLabel->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
    DPasswordEdit *codeedit = new DPasswordEdit();
    codeedit->setPlaceholderText(tr("Please input the auth code"));
    codeedit->setFixedSize(220,36);
    codeLayout->setSpacing(16);
    codeLayout->addStretch();
    codeLayout->addWidget(codeLabel);
    codeLayout->addWidget(codeedit);
    codeLayout->addSpacing(44);
    codeLayout->addStretch();
    connect(codeedit, &DPasswordEdit::textChanged, [this](const QString&) {clearHits();});

    QHBoxLayout *codeHitsLayout = new QHBoxLayout;
    data->codeHits = new DLabel();
    data->codeHits->setStyleSheet(hitsSytleSheet);
    data->codeHits->setFixedSize(220,36);
    codeHitsLayout->setSpacing(16);
    codeHitsLayout->addStretch();
    codeHitsLayout->addSpacing(140);
    codeHitsLayout->addWidget(data->codeHits);
    codeHitsLayout->addSpacing(34);
    codeHitsLayout->addStretch();

    QHBoxLayout *actionLayout = new QHBoxLayout;
    DPushButton *actionBt = new DPushButton(tr("Next"));
    connect(actionBt, SIGNAL(clicked(bool)), this, SLOT(startVerfiy()));
    connect(actionBt, SIGNAL(clicked(bool)), this, SLOT(clearHits()));
    actionBt->setFixedSize(120, 36);
    actionLayout->addWidget(actionBt);

    data->spinner = new DSpinner;
    actionLayout->addWidget(data->spinner);
    data->spinner->hide();

    layout->addSpacing(70);
    layout->addLayout(ipLayout);
    layout->addLayout(ipHitsLayout);
    layout->addLayout(codeLayout);
    layout->addLayout(codeHitsLayout);
    layout->addSpacing(24);
    layout->addLayout(actionLayout);
    layout->addStretch();

    QWidget *authWidget = new QWidget;
    authWidget->setLayout(layout);

    data->ipedit = ipedit;
    data->authcodeEdit = codeedit;
    data->actBt = actionBt;

    return authWidget;
}


void ConfigDialog::startVerfiy() {
    data->worker->ipaddr = data->ipedit->text();
    data->worker->authcode = data->authcodeEdit->text();
    data->actBt->hide();
    data->spinner->start();
    data->spinner->show();
    data->ipedit->deselect();
    data->authcodeEdit->deselect();

    emit verfiyStart();
}

void ConfigDialog::finishVerfiy(int errcode) {
    switch (errcode) {
    case ConfigDialogInnter::Worker::Success:
        break;
    case ConfigDialogInnter::Worker::InvaildAddress:
        data->ipHits->setText(tr("Invalid IP address"));
        data->actBt->show();
        data->spinner->hide();
        data->spinner->stop();
        return;
    case ConfigDialogInnter::Worker::InvaildAuthcode:
        data->codeHits->setText(tr("Invalid authorization code"));
        data->actBt->show();
        data->spinner->hide();
        data->spinner->stop();
        return;
    case ConfigDialogInnter::Worker::InvaildPrinterList:
        data->ipHits->setText(tr("Invalid IP address"));
        data->actBt->show();
        data->spinner->hide();
        data->spinner->stop();
        return;
    default:
        break;
    }


    std::vector<Printer> printers = data->worker->printers;

    if (0 == printers.size()) {
        data->actBt->show();
        data->spinner->hide();
        data->spinner->stop();

        QStringList buttons, buttonTexts;
        buttons << "Ok";
        buttonTexts << tr("OK");
        DMessageBox msg(":/View/Resource/config-helper.svg",
                        tr("No printer available, please try to connect to other print server"),
                        "", buttons, buttonTexts, this);
        msg.exec();
        return;
    }

    DListWidget *contentsWidget = new DListWidget;
    contentsWidget->setFlow(QListView::TopToBottom);
    contentsWidget->setMovement(QListView::Static);
    contentsWidget->setViewMode(QListView::ListMode);
    contentsWidget->setSpacing(0);

    QMap<QString, cloudprint::Printer> existPrinters = CUPSClient::listPrinter();
    foreach (Printer printer, printers) {
        qDebug()<<"printer.Status"<<QString("0x%1").arg(printer.Status , 8, 16, QLatin1Char('0'));
        PrinterWidget *myItem = new PrinterWidget(printer, existPrinters.contains(QString::fromStdString(printer.Addr)));
        QListWidgetItem *item = new QListWidgetItem();
        item->setSizeHint(myItem->size());
        contentsWidget->addItem(item);
        contentsWidget->setItemWidget(item,myItem);
    }
    data->title->setText(QString(tr("Select the printer to add (%1)")).arg(data->ipedit->text()));
    data->authWidget->hide();
    data->listWidget = contentsWidget;
    data->layout->addWidget(contentsWidget);

//    disconnect(data->worker, SIGNAL(listPrinterFinish(int)), this, SLOT(finishVerfiy(int)));
//    connect(data->worker, SIGNAL(listPrinterFinish(int)), this, SLOT(refreshPrinterList(int)));
//    QTimer::singleShot(200, data->worker, SLOT(doListPrinter()));
}

void ConfigDialog::refreshPrinterList(int /*code*/) {
//    DListWidget *contentsWidget = new DListWidget;
//    contentsWidget->setFlow(QListView::TopToBottom);
//    contentsWidget->setMovement(QListView::Static);
//    contentsWidget->setViewMode(QListView::ListMode);
//    contentsWidget->setSpacing(0);

//    std::vector<cloudprint::Printer> printers = data->worker->printers;
//    QMap<QString, cloudprint::Printer> existPrinters = CUPSClient::listPrinter();
//    foreach (cloudprint::Printer printer, printers) {
//        qDebug()<<"printer.Status"<<printer.Status;
//        printer.Status = cloudprint::PrinterStatus::Available;
//        if (existPrinters.contains(QString::fromStdString(printer.Addr))) {
//            printer.Status = cloudprint::PrinterStatus::Existed;
//        }
//        PrinterWidget *myItem = new PrinterWidget(printer);
//        QListWidgetItem *item = new QListWidgetItem();
//        item->setSizeHint(myItem->size());
//        contentsWidget->addItem(item);
//        contentsWidget->setItemWidget(item,myItem);
//    }
//    data->listWidget->hide();
//    data->listWidget->deleteLater();
//    data->listWidget = contentsWidget;
//    data->authWidget->hide();
//    data->layout->addWidget(contentsWidget);

//    QTimer::singleShot(200, data->worker, SLOT(doListPrinter()));
}

bool ConfigDialog::eventFilter(QObject *obj, QEvent *event)
{
    if (event->type() == QEvent::KeyPress) {
        QKeyEvent *keyEvent = static_cast<QKeyEvent *>(event);
        if (keyEvent->key() == Qt::Key_F1) {
            qDebug() << "show manual for help...";
            if (NULL == data->dManual) {
                data->dManual =  new QProcess(this);
                const QString pro = "dman";
                const QStringList args("deepin-cloud-print");
                connect(data->dManual, SIGNAL(finished(int)), this, SLOT(manualClose(int)));
                data->dManual->start(pro, args);
            }
            return true;
        }
    }
    // standard event processing
    return QObject::eventFilter(obj, event);
}

void ConfigDialog::manualClose(int) {
    data->dManual->deleteLater();
    data->dManual = NULL;
}
