#include "PrinterWidget.h"

#include <QIcon>
#include <QTimer>
#include <QProcess>
#include <QLabel>
#include <QPainter>
#include <QThread>
#include <QHBoxLayout>
#include <QVBoxLayout>

#include "DImageButton.h"
#include "DSpinner.h"
#include "DLabel.h"
#include "../CUPSClient.h"
#include "../CloudPrintClient.h"

#include <QDebug>

static void drawChecked(QLabel *btIcon) {
    QImage tmp(btIcon->pixmap()->toImage());
    QPixmap checked(":/View/Resource/check.png");
    QPainter painter(&tmp);
    QPoint p1;
    p1.setX(tmp.width()-checked.width());
    p1.setY(checked.height() + (checked.height()-tmp.height())/2);
    painter.drawPixmap(p1,checked);
    btIcon->setPixmap(QPixmap::fromImage(tmp));
}

void PrinterWidgetInnter::Worker::doAddPrinter() {
    QThread::sleep(2);
    if (CUPSClient::addPrinter(printer)) {
        emit addPrinterFinish(Success);
    }
    emit addPrinterFinish(Failed);
}

PrinterWidget::PrinterWidget(Printer& p, bool added, QWidget *parent)
    : QWidget(parent), printer(p) {
    exist = false;
    QHBoxLayout *layout = new QHBoxLayout;

    QPixmap ppixmap(":/View/Resource/could-printer.svg");
    btIcon = new QLabel;
    btIcon->setPixmap(ppixmap);
    btIcon->resize(32,32);

    QVBoxLayout *textLayout = new QVBoxLayout;
    textLayout->setSpacing(0);

    QHBoxLayout *titleLayout = new QHBoxLayout;
    titleLayout->setSpacing(0);
    QLabel *printerName = new QLabel(printer.ID.c_str());
    printerName->setStyleSheet("color: #000000; font-size: 12px");
    printerAdded = new DLabel(tr("(Added)"));
    printerAdded->hide();
    titleLayout->addWidget(printerName);
    titleLayout->addWidget(printerAdded);

    QLabel *printerStatus = new QLabel();
    if (printer.cJobs > 0) {
        printerStatus->setText(QString(tr("%1 print jobs are being processed")).arg(printer.cJobs));
    } else {
        printerStatus->setText(tr("Printer is idle"));
    }

//    switch (printer.localStatus()) {
//    case Printer::Idle:
//        break;
//    case Printer::Processing:
//    case Printer::OutWork:
//    default:
//        printerStatus->setText(QString(tr("Printer is out of work (Reasons: %1)")).arg(printer.statusString()));
//        break;
//    }

    printerStatus->setStyleSheet("color: #525252; font-size: 10px");
    textLayout->addLayout(titleLayout);
    textLayout->addWidget(printerStatus);

    if (added) {
        exist = true;
        drawChecked(btIcon);
        printerAdded->show();
        addBt = new DImageButton(":/View/Resource/set_normal.png", ":/View/Resource/set_hover.png",":/View/Resource/set_press.png");
        connect(addBt, SIGNAL(clicked()), this, SLOT(configPrinter()));
    } else {
        QThread* thread = new QThread;
        worker = new PrinterWidgetInnter::Worker();
        worker->printer = printer;
        worker->moveToThread(thread);
        connect(worker, SIGNAL(addPrinterFinish(int)), this, SLOT(finishAddPrinter(int)));
        connect(this, SIGNAL(addPrinterBegin()), worker, SLOT(doAddPrinter()));
        thread->start();

        addBt = new DImageButton(":/View/Resource/add_normal.png", ":/View/Resource/add_hover.png",":/View/Resource/add_press.png");
        connect(addBt, SIGNAL(clicked()), this, SLOT(addPrinter()));
    }

    addSpipper= new DSpinner(14);
    addSpipper->hide();

    QWidget *btWidget = new QWidget;
    QVBoxLayout *btLayout = new QVBoxLayout;
    btLayout->setSpacing(0);
    btLayout->setContentsMargins(0,0,0,0);
    btLayout->addStretch();
    btLayout->addWidget(addBt);
    btLayout->addWidget(addSpipper);
    btLayout->setAlignment(addSpipper, Qt::AlignCenter);
    btLayout->addStretch();
    btWidget->setLayout(btLayout);
    btWidget->setFixedWidth(24);

    layout->addSpacing(30);
    layout->addWidget(btIcon);
    layout->addSpacing(16);
    layout->addLayout(textLayout);
    layout->addStretch();
    layout->addWidget(btWidget);
    layout->addSpacing(20);
    setLayout(layout);
    setFixedSize(500,60);

    QTimer::singleShot(2000, this, SLOT(checkStatus()));
}

void PrinterWidget::checkStatus() {
    cups_dest_t* dest = CUPSClient::getPrinter(printer);
    bool cupsExist = (NULL != dest);

    if (exist != cupsExist) {
        addBt->disconnect(this);
        if (cupsExist) {
            drawChecked(btIcon);
            printerAdded->show();
            addBt->setNormalPic(":/View/Resource/set_normal.png");
            addBt->setHoverPic(":/View/Resource/set_hover.png");
            addBt->setPressPic(":/View/Resource/set_press.png");
            connect(addBt, SIGNAL(clicked()), this, SLOT(configPrinter()));
        } else {
            printerAdded->hide();
            btIcon->setPixmap(QPixmap(":/View/Resource/could-printer.svg"));
            addBt->setNormalPic(":/View/Resource/add_normal.png");
            addBt->setHoverPic(":/View/Resource/add_hover.png");
            addBt->setPressPic(":/View/Resource/add_press.png");
            connect(addBt, SIGNAL(clicked()), this, SLOT(addPrinter()));
        }
    }
    exist = cupsExist;
    QTimer::singleShot(2000, this, SLOT(checkStatus()));
}

void PrinterWidget::addPrinter() {
    qDebug()<<QString::fromStdString(printer.ID)<< "addPrinter";
    emit addPrinterBegin();
    addBt->hide();
    addSpipper->show();
    addSpipper->start();
}


void PrinterWidget::finishAddPrinter(int errcode) {
    addSpipper->stop();
    addSpipper->hide();
    addBt->show();
    if (PrinterWidgetInnter::Worker::Success == errcode) {
        drawChecked(btIcon);
        printerAdded->show();
        addBt->setNormalPic(":/View/Resource/set_normal.png");
        addBt->setHoverPic(":/View/Resource/set_hover.png");
        addBt->setPressPic(":/View/Resource/set_press.png");
        addBt->disconnect(this);
        connect(addBt, SIGNAL(clicked()), this, SLOT(configPrinter()));
        exist = true;
   }
   emit addPrinterFinish();
}

void PrinterWidget::configPrinter() {
    qDebug()<<"call configPrinter";
    QProcess::startDetached("system-config-printer");
}

void PrinterWidget::paintEvent(QPaintEvent *) {
    QPainter painter(this);
    painter.setRenderHint(QPainter::Antialiasing);

    QRect rect = this->rect();
    QPoint bottomLeft(rect.x() + 30, rect.y()  + rect.height());
    QPoint bottomRight(rect.x() + rect.width() - 20, rect.y() + rect.height());

    QPen borderPen(QColor(0, 0, 0, 255*0.15));
    painter.setPen(borderPen);
    painter.drawLine(bottomLeft, bottomRight);
}
