#include "DUI.h"
#include "DAnimation.h"

#include <QTimer>
#include <QPainter>

DAnimation::DAnimation(QWidget *parent):
    QWidget(parent)
{
    init();
}

DAnimation::DAnimation(const QStringList &sourcesList, QWidget *parent):
    QWidget(parent)
{
    init();

    m_SourceList = sourcesList;
    QString firstFrame = m_SourceList[m_FrameIndex];
    this->setFixedSize(QPixmap(firstFrame).size());
}

void DAnimation::init() {
    DUI_INIT_STYLESHEET(DAnimation);

    m_Timer=new QTimer(this);
    connect(m_Timer,SIGNAL(timeout()),this,SLOT(nextFrame()));
    m_FrameIndex = 0;
}

void DAnimation::nextFrame() {
    if (0 == m_SourceList.length()) {
        return;
    }
    ++m_FrameIndex;
    if (m_FrameIndex == m_SourceList.length()) {
        emit finish();
        m_FrameIndex = 0;
    }
    this->repaint();
}

void DAnimation::paintEvent(QPaintEvent *e)
{
    if (m_SourceList.length()) {
        QPainter painter(this);
        painter.drawPixmap(0, 0, QPixmap(m_SourceList[m_FrameIndex]).scaled(size()));
    }
    QWidget::paintEvent(e);
}

void DAnimation::start() {
    m_Timer->start(30);
}

void DAnimation::stop() {
    if (0 == m_SourceList.length()) {
        return;
    }
    m_FrameIndex = 0;
    m_Timer->stop();
}

