#include "DTitle.h"

#include <QLabel>
#include <QPainter>
#include <QHBoxLayout>
#include <QPushButton>
#include <QApplication>

#include "DUI.h"

class DTitleData {
public:
    QLabel      *title;
    QPushButton *miniButton;
    QPushButton *maxButton;
    QPushButton *closeButton;
};

DTitle::DTitle(QWidget *parent) : QWidget(parent) {
    DUI_INIT_STYLESHEET(DTitle);

    data = new DTitleData;

    data->closeButton = new QPushButton(this);
    data->closeButton->setObjectName("CloseButton");
    data->closeButton->setFixedSize(28, 24);
    data->closeButton->setAttribute(Qt::WA_NoMousePropagation);
    data->closeButton->setFocusPolicy(Qt::NoFocus);
    connect(data->closeButton, SIGNAL(clicked(bool)),
            this, SIGNAL(closed()));

    data->miniButton = new QPushButton(this);
    data->miniButton->setObjectName("MinimizeButton");
    data->miniButton->setFixedSize(28, 24);
    data->miniButton->setAttribute(Qt::WA_NoMousePropagation);
    data->miniButton->setFocusPolicy(Qt::NoFocus);
    connect(data->miniButton, SIGNAL(clicked(bool)),
            this,  SIGNAL(minimised()));

    data->title = new QLabel("Unkown Application");
    data->title->setStyleSheet("font-size: 14px");

    QHBoxLayout *layout = new QHBoxLayout;
    layout->setContentsMargins(24, 0, 10, 0);
    layout->addWidget(data->title);
    layout->addStretch();

    layout->addWidget(data->miniButton);
    layout->addWidget(data->closeButton);

    setLayout(layout);
}

void DTitle::setText(const QString& text){
    data->title->setText(text);
}

DTitle::~DTitle(){
    delete data;
}

void DTitle::paintEvent(QPaintEvent *) {
    QPainter painter(this);
    painter.setRenderHint(QPainter::Antialiasing);

    QRect rect = this->rect();
    QPoint bottomLeft(rect.x(), rect.y() + rect.height());
    QPoint bottomRight(rect.x() + rect.width(), rect.y() + rect.height());

    QPen borderPen(QColor(0, 0, 0, 255*0.15));
    painter.setPen(borderPen);
    painter.drawLine(bottomLeft, bottomRight);
}
