#include "DWindow.h"
#include "DUI.h"

#include <QPainter>
#include <QMouseEvent>
#include <QGraphicsDropShadowEffect>

#include <QX11Info>
#include <X11/Xlib.h>
#include <X11/Xatom.h>

QString g_ThemeName = "Light";

DWindow::DWindow(QWidget *parent) : QWidget(parent) {
    setAttribute(Qt::WA_TranslucentBackground, true);
    setWindowFlags(Qt::FramelessWindowHint | Qt::Window);
    setFocusPolicy(Qt::NoFocus);
    m_Shadow= NULL;
    m_Radius = 4;
}

void DWindow::setShadowHits(int glowRadius) {
    const int maxInterLength = 40;
    QString glowRadiusString = QString("%1").arg(glowRadius);
    unsigned char value[maxInterLength];
    memset(value, 0, maxInterLength);
    memcpy(value, glowRadiusString.toStdString().c_str(), glowRadiusString.size());

    Atom shadowAtom;

    shadowAtom = XInternAtom(QX11Info::display(), "DEEPIN_WINDOW_SHADOW", False);
    if (shadowAtom == None) {
          qDebug()<<"Failed to create atom with name DEEPIN_WINDOW_SHADOW";
          return;
    }

    XChangeProperty(QX11Info::display(),
                             this->winId(),
                             shadowAtom,
                             XA_STRING,
                             8,
                             PropModeReplace,
                             value,
                             glowRadiusString.size());

    m_GlowRadius = glowRadius;
    m_Shadow = new QGraphicsDropShadowEffect(this);
    m_Shadow->setOffset(0, 0);
    m_Shadow->setBlurRadius(m_GlowRadius);
    m_Shadow->setColor(DUI::ShadowColor);
    this->setGraphicsEffect(m_Shadow);
}

void DWindow::showEvent(QShowEvent *e) {
    if (!m_Shadow) {
        setShadowHits(8);
    }
    QWidget::showEvent(e);
}

void DWindow::paintEvent(QPaintEvent *) {
    QPainter painter(this);
    painter.setRenderHint(QPainter::Antialiasing);

    QRect rect = this->rect().marginsRemoved(QMargins(m_GlowRadius, m_GlowRadius, m_GlowRadius, m_GlowRadius));
    QPoint topLeft(rect.x(), rect.y());
    QPoint bottomRight(rect.x() + rect.width(), rect.y() + rect.height());
    QPainterPath border;
    border.addRoundedRect(rect, m_Radius, m_Radius);

    QLinearGradient linearGradient(topLeft, QPoint(topLeft.x(), bottomRight.y()));
    linearGradient.setColorAt(0.0, DUI::BackgroundTopColor);
    linearGradient.setColorAt(0.2, DUI::BackgroundBottonColor);
    linearGradient.setColorAt(1.0, DUI::BackgroundBottonColor);

    QPen borderPen(DUI::BorderColor);
    painter.setBrush(QBrush(linearGradient));
    painter.strokePath(border, borderPen);
    painter.fillPath(border, QBrush(linearGradient));
}

void DWindow::mousePressEvent(QMouseEvent *event) {
    QRect rect = this->rect().marginsRemoved(QMargins(m_GlowRadius, m_GlowRadius, m_GlowRadius, m_GlowRadius));
    if (rect.contains(event->pos())) {
        if(event->button() == Qt::LeftButton) {
            m_MousePressed = true;
            m_LastMousePos = event->globalPos() - this->pos();
        }
    }
}

void DWindow::mouseMoveEvent(QMouseEvent *event) {
    if(m_MousePressed) {
        move(event->globalPos() - m_LastMousePos);
    }
}

void DWindow::mouseReleaseEvent(QMouseEvent *event) {
    m_MousePressed = false;
    event->accept();
}
