#include "DBaseDialog.h"

#include "DUI.h"

#include <QPushButton>
#include <QLabel>
#include <QPixmap>
#include <QButtonGroup>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QDebug>
#include <QCloseEvent>
#include <QApplication>
#include <QDesktopWidget>


DBaseDialog::DBaseDialog(QWidget *parent):DMovabelDialog(parent)
{
    m_closeButton = new QPushButton(this);
    m_closeButton->setObjectName("CloseButton");
    m_closeButton->setFixedSize(25, 25);
    m_closeButton->setAttribute(Qt::WA_NoMousePropagation);
    m_messageLabelMaxWidth = qApp->desktop()->availableGeometry().width() / 2 -
            100 - 2 * m_closeButton->width();
}

void DBaseDialog::initUI(const QString& icon,
            QWidget *widget,
            const QStringList& buttonKeys,
            const QStringList& buttonTexts) {
    m_icon = icon;

    m_buttonKeys = buttonKeys;
    m_buttonTexts = buttonTexts;

    QFrame* contentFrame = new QFrame;
    contentFrame->setObjectName("ContentFrame");

    m_iconLabel = new QLabel;
    m_iconLabel->setFixedSize(48, 48);
    setIcon(m_icon);

    m_buttonGroup = new QButtonGroup;
    QHBoxLayout* buttonLayout = new QHBoxLayout;
    foreach (QString label, m_buttonKeys) {
        int index = m_buttonKeys.indexOf(label);
        QPushButton* button = new QPushButton(label);
        button->setObjectName("ActionButton");
        button->setAttribute(Qt::WA_NoMousePropagation);
        button->setFixedHeight(28);
        m_buttonGroup->addButton(button, index);
        buttonLayout->addWidget(button);
        if (index < m_buttonKeys.length() - 1){
            QLabel* label = new QLabel;
            label->setObjectName("VLine");
            label->setFixedWidth(1);
            buttonLayout->addWidget(label);
        }
    }

    setButtonTexts(m_buttonTexts);
    buttonLayout->setSpacing(0);
    buttonLayout->setContentsMargins(0, 0, 0, 0);

    QVBoxLayout* iconLayout = new QVBoxLayout;
    iconLayout->addWidget(m_iconLabel, Qt::AlignCenter);
    iconLayout->setContentsMargins(20, 14, 20, 14);

    QHBoxLayout* topLayout = new QHBoxLayout;
    topLayout->setSpacing(0);
    topLayout->setContentsMargins(0, 0, 0, 0);
    topLayout->addLayout(iconLayout);
    topLayout->addWidget(widget);
    topLayout->addStretch();

    QVBoxLayout* mainLayout = new QVBoxLayout;
    mainLayout->addStretch();
    mainLayout->addLayout(topLayout);
    mainLayout->addStretch();
    mainLayout->addLayout(buttonLayout);
    mainLayout->setSpacing(0);
    mainLayout->setContentsMargins(0, 0, 0, 0);
    contentFrame->setLayout(mainLayout);

    QHBoxLayout* contentlayout = new QHBoxLayout;
    contentlayout->addWidget(contentFrame);
    contentlayout->setContentsMargins(5, 5, 5, 5);
    setLayout(contentlayout);

    setMinimumSize(m_defaultWidth, m_defaultHeight);
    resize(m_defaultWidth, m_defaultHeight);
    DUI_INIT_STYLESHEET(DDialog)
    initConnect();
}


void DBaseDialog::initConnect(){
    connect(m_buttonGroup, SIGNAL(buttonClicked(int)), this, SLOT(handleButtonsClicked(int)));
    connect(m_closeButton, SIGNAL(clicked()), this, SLOT(close()));
}

QButtonGroup* DBaseDialog::getButtonsGroup(){
    return m_buttonGroup;
}

QString DBaseDialog::getIcon(){
    return m_icon;
}

void DBaseDialog::setIcon(const QString &icon){
    m_icon = icon;
    m_iconLabel->setPixmap(QPixmap(m_icon).scaled(m_iconLabel->size()));
}

QStringList DBaseDialog::getButtons(){
    return m_buttonKeys;
}

void DBaseDialog::setButtons(const QStringList &buttons){
    m_buttonKeys = buttons;
}

QStringList DBaseDialog::getButtonTexts(){
    return m_buttonTexts;
}

void DBaseDialog::setButtonTexts(const QStringList& buttonTexts){
    if (buttonTexts.length() != m_buttonKeys.length()){
        return;
    }
    m_buttonTexts = buttonTexts;
    for (int i = 0; i < m_buttonGroup->buttons().length(); i++) {
        QPushButton* button = reinterpret_cast<QPushButton*>(m_buttonGroup->buttons().at(i));
        button->setText(m_buttonTexts.at(i));
    }
}

void DBaseDialog::handleButtonsClicked(int id){
    close();
    int index = m_buttonGroup->buttons().indexOf(m_buttonGroup->button(id));
    if (m_buttonKeys.length() > index){
        QString key = m_buttonKeys.at(index);
        emit buttonClicked(key);
        emit buttonClicked(index);
    }
}

void DBaseDialog::handleKeyEnter(){

}

void DBaseDialog::closeEvent(QCloseEvent *event){
    emit aboutToClose();
    DMovabelDialog::closeEvent(event);
    emit closed();
}


void DBaseDialog::resizeEvent(QResizeEvent *event){
    m_closeButton->move(width() - m_closeButton->width() - 4, 4);
    m_closeButton->raise();
    moveCenter();
    DMovabelDialog::resizeEvent(event);
}

DBaseDialog::~DBaseDialog()
{

}

