#include "DMovableDialog.h"
#include "../DUI.h"

#include <QMouseEvent>
#include <QApplication>
#include <QDesktopWidget>
#include <QPushButton>
#include <QResizeEvent>
#include <QPainter>
#include <QWidget>
#include <QDebug>

#include <QGraphicsDropShadowEffect>

#include <QX11Info>
#include <X11/Xlib.h>
#include <X11/Xatom.h>

void DMovabelDialog::setShadowHits(int glowRadius) {
    const int maxInterLength = 40;
    QString glowRadiusString = QString("%1").arg(glowRadius);
    unsigned char value[maxInterLength];
    memset(value, 0, maxInterLength);
    memcpy(value, glowRadiusString.toStdString().c_str(), glowRadiusString.size());

    Atom shadowAtom;

    shadowAtom = XInternAtom(QX11Info::display(), "DEEPIN_WINDOW_SHADOW", False);
    if (shadowAtom == None) {
          qDebug()<<"Failed to create atom with name DEEPIN_WINDOW_SHADOW";
          return;
    }

    XChangeProperty(QX11Info::display(),
                             this->winId(),
                             shadowAtom,
                             XA_STRING,
                             8,
                             PropModeReplace,
                             value,
                             glowRadiusString.size());

    m_GlowRadius = glowRadius;
    m_Shadow = new QGraphicsDropShadowEffect(this);
    m_Shadow->setOffset(0, 0);
    m_Shadow->setBlurRadius(m_GlowRadius);
    m_Shadow->setColor(DUI::ShadowColor);
    this->setGraphicsEffect(m_Shadow);
}

DMovabelDialog::DMovabelDialog(QWidget *parent):QDialog(parent)
{
    m_Shadow = NULL;
    m_Radius = 4;
    setWindowFlags(Qt::FramelessWindowHint | Qt::Dialog);
    setAttribute(Qt::WA_TranslucentBackground);
}

void DMovabelDialog::setMovableHeight(int height){
    m_movableHeight = height;
}

void DMovabelDialog::moveCenter(){
    QRect qr = frameGeometry();
    QPoint cp;
    if (parent()){
        cp = static_cast<QWidget*>(parent())->geometry().center();
    }else{
        cp = qApp->desktop()->availableGeometry().center();
    }
    qr.moveCenter(cp);
    move(qr.topLeft());
}

void DMovabelDialog::moveTopRight(){
    QRect pRect;
    if (parent()){
        pRect = static_cast<QWidget*>(parent())->geometry();
    }else{
        pRect = qApp->desktop()->availableGeometry();
    }
    int x = pRect.width() - width();
    move(QPoint(x, 0));
}

void DMovabelDialog::moveTopRightByRect(QRect rect){
    int x = rect.x() + rect.width() - width();
    move(QPoint(x, 0));
}

void DMovabelDialog::moveCenterByRect(QRect rect){
    QRect qr = frameGeometry();
    qr.moveCenter(rect.center());
    move(qr.topLeft());
}

void DMovabelDialog::mousePressEvent(QMouseEvent *event)
{
    if(event->button() & Qt::LeftButton)
    {
        m_dragPosition = event->globalPos() - frameGeometry().topLeft();
    }
    QDialog::mousePressEvent(event);
}

void DMovabelDialog::mouseReleaseEvent(QMouseEvent *event)
{
    QDialog::mouseReleaseEvent(event);
}

void DMovabelDialog::mouseMoveEvent(QMouseEvent *event)
{
    move(event->globalPos() - m_dragPosition);
    QDialog::mouseMoveEvent(event);
}

void DMovabelDialog::resizeEvent(QResizeEvent *event){
    QDialog::resizeEvent(event);
}

void DMovabelDialog::paintEvent(QPaintEvent *event){
    QPainter painter(this);
    painter.setPen(QPen(QColor(255, 255, 255, 51), 2));
    painter.setBrush(QColor(0, 0 , 0, 204));
    painter.setRenderHint(QPainter::Antialiasing, true);
    QRect r(4, 4, width() - 8, height() - 8);
    painter.drawRoundedRect(r, 2, 2, Qt::RelativeSize);
    QDialog::paintEvent(event);
}

void DMovabelDialog::showEvent(QShowEvent *e) {
    if (!m_Shadow) {
        setShadowHits(4);
    }
    QDialog::showEvent(e);
}

DMovabelDialog::~DMovabelDialog()
{

}

