package main

import (
	"encoding/json"
	"pkg.deepin.io/cloudprint/rpc/print"
	"strconv"
	"strings"
)

func parsePageSet(pageset string) print.PrintPageSet {
	switch pageset {
	case "even":
		return print.PrintPageSet_Even
	case "odd":
		return print.PrintPageSet_Odd
	default:
		return print.PrintPageSet_All
	}
}

func parseNumberUpLayout(nul string) print.NumberUpLayout {
	switch nul {
	case "lrtb":
		return print.NumberUpLayout_LRTB
	case "btlr":
		return print.NumberUpLayout_BTLR
	case "rlbt":
		return print.NumberUpLayout_RLBT
	case "tbrl":
		return print.NumberUpLayout_TBRL
	case "lrbt":
		return print.NumberUpLayout_LRBT
	case "btrl":
		return print.NumberUpLayout_BTRL
	case "rltb":
		return print.NumberUpLayout_RLTB
	case "tblr":
		return print.NumberUpLayout_TBLR
	default:
		return print.NumberUpLayout_LRTB
	}
}

func parseOutputOrder(outputorder string) print.OutputOrder {
	switch outputorder {
	case "Reverse":
		return print.OutputOrder_Reverse
	default:
		return print.OutputOrder_Normal
	}
}

func setArg2(op *print.Option, k, v string) {
	switch k {
	case "number-up":
		numberUp, _ := strconv.Atoi(v)
		op.NumberUp = int32(numberUp)
	case "page-set":
		op.PageSet = parsePageSet(v)
	case "number-up-layout":
		op.Layout = parseNumberUpLayout(v)
	case "PageSize":
		op.PageSize = v
	case "OutputOrder":
		op.Order = parseOutputOrder(v)
	case "Resolution":
		op.Resilution = v
	}
}

func setArg1(op *print.Option, k string) {
	switch k {
	case "Collate":
		op.Collate = true
	}
}

func setArg(op *print.Option, kv []string) {
	switch len(kv) {
	case 1:
		setArg1(op, kv[0])
	case 2:
		setArg2(op, kv[0], kv[1])
	}
}

//1 number-up=2 page-set=even number-up-layout=btlr PageSize=A4 OutputOrder=Reverse Collate Resolution=300dpi
//1 number-up-layout=btlr PageSize=ISOB3 OutputOrder=Reverse Resolution=300dpi number-up=16
// 1 number-up=2 page-set=even number-up-layout=rltb PageSize=AnsiB OutputOrder=Reverse Resolution=300dpi
//number-up=2 page-set=even number-up-layout=btlr PageSize=A4 OutputOrder=Reverse Collate Resolution=300dpi
func defaultOption() print.Option {
	return print.Option{
		Copyies:    1,
		NumberUp:   1,
		Layout:     print.NumberUpLayout_RLTB,
		PageSize:   "A4",
		PageSet:    print.PrintPageSet_All,
		Order:      print.OutputOrder_Normal,
		Resilution: "300dpi",
		Collate:    false,
	}
}

func parseCUPSOptions(options string) print.Option {
	option := defaultOption()
	args := strings.Split(options, " ")
	for _, arg := range args {
		setArg(&option, strings.Split(arg, "="))
	}
	return option
}

func debugOption(op *print.Option) string {
	data, _ := json.Marshal(op)
	return string(data)
}
