namespace cpp cloudprint

exception Exception {
  1: i32 Code,
  2: string Description
}

typedef string  Timestamp
typedef string  PrinterID
typedef string  JobID

enum PrinterStatus {
    Available = 1,
    Existed = 0,
}

const i32 PrinterStatusPaused               = 0x00000001,
const i32 PrinterStatusError                = 0x00000002,
const i32 PrinterStatusPendingDeletion      = 0x00000004,
const i32 PrinterStatusPaperJam             = 0x00000008,
const i32 PrinterStatusPaperOut             = 0x00000010,
const i32 PrinterStatusManualFeed           = 0x00000020,
const i32 PrinterStatusPaperProblem         = 0x00000040,
const i32 PrinterStatusOffline              = 0x00000080,
const i32 PrinterStatusIoActive             = 0x00000100,
const i32 PrinterStatusBusy                 = 0x00000200,
const i32 PrinterStatusPrinting             = 0x00000400,
const i32 PrinterStatusOutputBinFull        = 0x00000800,
const i32 PrinterStatusNotAvailable         = 0x00001000,
const i32 PrinterStatusWaiting              = 0x00002000,
const i32 PrinterStatusProcessing           = 0x00004000,
const i32 PrinterStatusInitializing         = 0x00008000,
const i32 PrinterStatusWarmingUp            = 0x00010000,
const i32 PrinterStatusTonerLow             = 0x00020000,
const i32 PrinterStatusNoToner              = 0x00040000,
const i32 PrinterStatusPagePunt             = 0x00080000,
const i32 PrinterStatusUserIntervention     = 0x00100000,
const i32 PrinterStatusOutOfMemory          = 0x00200000,
const i32 PrinterStatusDoorOpen             = 0x00400000,
const i32 PrinterStatusServerUnknown        = 0x00800000,
const i32 PrinterStatusPowerSave            = 0x01000000,
const i32 PrinterStatusServerOffline        = 0x02000000,
const i32 PrinterStatusDriverUpdateNeeded   = 0x04000000,

const i32 JobStatusPaused               = 0x00000001,
const i32 JobStatusError                = 0x00000002,
const i32 JobStatusDeleting             = 0x00000004,
const i32 JobStatusSpooling             = 0x00000008,
const i32 JobStatusPrinting             = 0x00000010,
const i32 JobStatusOffline              = 0x00000020,
const i32 JobStatusPaperout             = 0x00000040,
const i32 JobStatusPrinted              = 0x00000080,
const i32 JobStatusDeleted              = 0x00000100,
const i32 JobStatusBlockedDevq          = 0x00000200,
const i32 JobStatusUserIntervention     = 0x00000400,
const i32 JobStatusRestart              = 0x00000800,
const i32 JobStatusComplete             = 0x00001000,


const i32 PrinterIdle        = 0x00000000,
const i32 PrinterProcessing  = 0x00000001,
const i32 PrinterError       = 0x00000002,

struct Printer {
  1: PrinterID ID,
  2: string    ServerName,
  3: string    ShareName,
  4: string    PortName,
  5: string    Addr,
  6: string    DriverName,
  7: string    Comment,
  8: string    Location,
  9: string    SepFile,
  10: string   PrintProcessor,
  11: string   Datatype,
  12: string   Parameters,
  13: i32      Attributes,
  14: i32      Priority,
  15: i32      DefaultPriority,
  16: i32      Status,
  17: i32      cJobs,
  18: i32      AveragePPM,
}

struct Job {
  1:  JobID      JobId,
  2:  string     PrinterName,
  3:  string     MachineName,
  4:  string     UserName,
  5:  string     Document,
  6:  string     Datatype,
  7:  string     StatusString,
  8:  i32        Status,
  9:  i32        Priority,
  10: i32        Position,
  11: i32        TotalPages,
  12: i32        PagesPrinted,
  13: Timestamp  Submitted,
}

// https://msdn.microsoft.com/en-us/library/cc244823.aspx
enum JobCommand {
  // Pause the print job.
  JobControlPause = 1,
  // Resume a paused print job.
  JobControlResume = 2,
  // Do not use. To delete a print job, use JobControlDelete.
  JobControlCancel = 3,
  // Restart the print job. A job can only be restarted if it was printing.
  JobControlRestart = 4,
  // Delete the print job.
  JobControlDelete = 5,
}


//1 number-up=2 page-set=even number-up-layout=btlr PageSize=A4 OutputOrder=Reverse Collate Resolution=300dpi
//1 number-up-layout=btlr PageSize=ISOB3 OutputOrder=Reverse Resolution=300dpi number-up=16
// 1 number-up=2 page-set=even number-up-layout=rltb PageSize=AnsiB OutputOrder=Reverse Resolution=300dpi
enum NumberUpLayout {
    //L->R, lengthwise
    LRTB = 0,
    //L->R, landspace
    BTLR = 1,
    //L->R, lengthwise, reverse
    RLBT = 2
    //L->R, landspace, reverse
    TBRL = 3,
    //R->L, lengthwise
    LRBT = 4,
    //R->L, landspace
    BTRL = 5,
    //R->L, lengthwise, reverse
    RLTB = 6,
    //R->L, landspace, reverse
    TBLR = 7,
}

enum PrintPageSet {
    All = 0,
    Odd = 1,
    Even = 2,
}

enum OutputOrder {
    Normal = 0,
    Reverse = 1,
}

struct Option {
    1: i32              Copyies,
    2: i32              NumberUp,
    3: NumberUpLayout   Layout,
    4: string           PageSize,
    5: PrintPageSet     PageSet,
    6: OutputOrder      Order,
    7: string           Resilution,
    8: bool             Collate,
}
